@@ -24,8 +24,10 @@ except ImportError:
     from functools import lru_cache as cache
 
 from contextlib import contextmanager
+from copy import copy
 from dataclasses import dataclass, field
 from enum import Enum
+from fnmatch import fnmatch
 from io import BytesIO
 from multiprocessing import Process, Queue
 from multiprocessing.connection import Connection
@@ -127,9 +129,13 @@ class BigQueryTable:
             setattr(table, option, value)
         return table
 
-    def as_dataset(self) -> bigquery.Dataset:
-        """Returns a Dataset instance for this table."""
-        return bigquery.Dataset(self.as_dataset_ref())
+    def as_dataset(self, **kwargs) -> bigquery.Dataset:
+        """Returns a Dataset instance for this dataset."""
+        dataset = bigquery.Dataset(self.as_dataset_ref())
+        config = {**self.default_dataset_options(), **kwargs}
+        for option, value in config.items():
+            setattr(dataset, option, value)
+        return dataset
 
     def create_table(
         self,
@@ -143,13 +149,25 @@ class BigQueryTable:
         table in a single method call. It is idempotent and will not create
         a new table if one already exists."""
         if not hasattr(self, "_dataset"):
-            self._dataset = client.create_dataset(self.as_dataset(), exists_ok=True)
+            try:
+                self._dataset = client.create_dataset(
+                    self.as_dataset(**kwargs["dataset"]), exists_ok=False
+                )
+            except Conflict:
+                dataset = client.get_dataset(self.as_dataset(**kwargs["dataset"]))
+                if dataset.location != kwargs["dataset"]["location"]:
+                    raise Exception(
+                        f"Location of existing dataset {dataset.dataset_id} ({dataset.location}) "
+                        f"does not match specified location: {kwargs['dataset']['location']}"
+                    )
+                else:
+                    self._dataset = dataset
         if not hasattr(self, "_table"):
             try:
                 self._table = client.create_table(
                     self.as_table(
-                        apply_transforms and self.ingestion_strategy is not IngestionStrategy.FIXED,
-                        **kwargs,
+                        apply_transforms and self.ingestion_strategy != IngestionStrategy.FIXED,
+                        **kwargs["table"],
                     )
                 )
             except Conflict:
@@ -178,6 +196,11 @@ class BigQueryTable:
             ),
         }
 
+    @staticmethod
+    def default_dataset_options() -> Dict[str, Any]:
+        """Returns the default dataset options for this dataset."""
+        return {"location": "US"}
+
     def __hash__(self) -> int:
         return hash((self.name, self.dataset, self.project, json.dumps(self.jsonschema)))
 
@@ -249,13 +272,15 @@ class BaseBigQuerySink(BatchSink):
         self.create_target(key_properties=key_properties)
         self.update_schema()
         self.merge_target: Optional[BigQueryTable] = None
+        self.overwrite_target: Optional[BigQueryTable] = None
+        # In absence of dedupe or overwrite candidacy, we append to the target table directly
+        # If the stream is marked for one of these strategies, we create a temporary table instead
+        # and merge or overwrite the target table with the temporary table after the ingest.
         if (
             key_properties
             and self.ingestion_strategy is IngestionStrategy.DENORMALIZED
-            and self.config.get("dedupe")
+            and self._is_upsert_candidate()
         ):
-            from copy import copy
-
             self.merge_target = copy(self.table)
             self.table = BigQueryTable(name=f"{self.table_name}__{int(time.time())}", **opts)
             self.table.create_table(
@@ -264,10 +289,71 @@ class BaseBigQuerySink(BatchSink):
                 expires=datetime.datetime.now() + datetime.timedelta(days=1),
             )
             time.sleep(2.5)  # Wait for eventual consistency
+        elif self._is_overwrite_candidate():
+            self.overwrite_target = copy(self.table)
+            self.table = BigQueryTable(name=f"{self.table_name}__{int(time.time())}", **opts)
+            self.table.create_table(
+                self.client,
+                self.apply_transforms,
+                expires=datetime.datetime.now() + datetime.timedelta(days=1),
+            )
+            time.sleep(2.5)  # Wait for eventual consistency
+
         self.global_par_typ = target.par_typ
         self.global_queue = target.queue
         self.increment_jobs_enqueued = target.increment_jobs_enqueued
 
+    def _is_upsert_candidate(self) -> bool:
+        """Determine if this stream is an upsert candidate based on user configuration."""
+        upsert_selection = self.config.get("upsert", False)
+        upsert_candidate = False
+        if isinstance(upsert_selection, list):
+            selection: str
+            for selection in upsert_selection:
+                invert = selection.startswith("!")
+                if invert:
+                    selection = selection[1:]
+                if fnmatch(self.stream_name, selection):
+                    upsert_candidate = True ^ invert
+        elif upsert_selection:
+            upsert_candidate = True
+        return upsert_candidate
+
+    def _is_overwrite_candidate(self) -> bool:
+        """Determine if this stream is an overwrite candidate based on user configuration."""
+        overwrite_selection = self.config.get("overwrite", False)
+        overwrite_candidate = False
+        if isinstance(overwrite_selection, list):
+            selection: str
+            for selection in overwrite_selection:
+                invert = selection.startswith("!")
+                if invert:
+                    selection = selection[1:]
+                if fnmatch(self.stream_name, selection):
+                    overwrite_candidate = True ^ invert
+        elif overwrite_selection:
+            overwrite_candidate = True
+        return overwrite_candidate
+
+    def _is_dedupe_before_upsert_candidate(self) -> bool:
+        """Determine if this stream is a dedupe before upsert candidate based on user configuration.
+        """
+        # TODO: we can enable this for `overwrite` too if we want but the purpose would
+        # be less for functional reasons (merge constraints) and more for convenience
+        dedupe_before_upsert_selection = self.config.get("dedupe_before_upsert", False)
+        dedupe_before_upsert_candidate = False
+        if isinstance(dedupe_before_upsert_selection, list):
+            selection: str
+            for selection in dedupe_before_upsert_selection:
+                invert = selection.startswith("!")
+                if invert:
+                    selection = selection[1:]
+                if fnmatch(self.stream_name, selection):
+                    dedupe_before_upsert_candidate = True ^ invert
+        elif dedupe_before_upsert_selection:
+            dedupe_before_upsert_candidate = True
+        return dedupe_before_upsert_candidate
+
     @property
     def table_name(self) -> str:
         """Returns the table name."""
@@ -316,15 +402,22 @@ class BaseBigQuerySink(BatchSink):
     )
     def create_target(self, key_properties: Optional[List[str]] = None) -> None:
         """Create the table in BigQuery."""
-        kwargs = {}
+        kwargs = {"table": {}, "dataset": {}}
+        # Table opts
         if key_properties and self.config.get("cluster_on_key_properties", False):
-            kwargs["clustering_fields"] = key_properties[:4]
+            kwargs["table"]["clustering_fields"] = key_properties[:4]
         partition_grain: str = self.config.get("partition_granularity")
         if partition_grain:
-            kwargs["time_partitioning"] = TimePartitioning(
+            kwargs["table"]["time_partitioning"] = TimePartitioning(
                 type_=PARTITION_STRATEGY[partition_grain.upper()],
                 field="_sdc_batched_at",
             )
+        # Dataset opts
+        location: str = self.config.get(
+            "location", BigQueryTable.default_dataset_options()["location"]
+        )
+        kwargs["dataset"]["location"] = location
+        # Create the table
         self.table.create_table(self.client, self.apply_transforms, **kwargs)
         if self.generate_view:
             self.client.query(
@@ -356,7 +449,7 @@ class BaseBigQuerySink(BatchSink):
             target = self.merge_target.as_table()
             date_columns = ["_sdc_extracted_at", "_sdc_received_at"]
             tmp, ctas_tmp = None, "SELECT 1 AS _no_op"
-            if self.config.get("non_unique_key_properties", True):
+            if self._is_dedupe_before_upsert_candidate():
                 # We can't use MERGE with a non-unique key, so we need to dedupe the temp table into
                 # a _SESSION scoped intermediate table.
                 tmp = f"{target.name}__tmp"
@@ -385,6 +478,18 @@ class BaseBigQuerySink(BatchSink):
             ).result()
             self.table = self.merge_target
             self.merge_target = None
+        elif self.overwrite_target is not None:
+            # We must overwrite the target table with the temp table.
+            # Do it in a transaction to avoid partial writes.
+            target = self.overwrite_target.as_table()
+            self.client.query(
+                f"DROP TABLE IF EXISTS {self.overwrite_target}; "
+                f"CREATE TABLE {self.overwrite_target} LIKE {self.table} AS "
+                f"SELECT * FROM {self.table};"
+                f"DROP TABLE IF EXISTS {self.table};"
+            ).result()
+            self.table = self.merge_target
+            self.merge_target = None
 
 
 class Denormalized:
